<?php

namespace App\Controllers;

use App\Models\JobOpeningsModel;
use App\Models\CandidateModel;
use App\Models\JobApplicationModel;

use CodeIgniter\API\ResponseTrait;


class Jobs extends BaseController
{
    use ResponseTrait;
    public function all()
    {
        $jobsModel = new JobOpeningsModel();
        $data = [];
        $openJobs = $jobsModel->where('status', 'active')->findAll();
        if(count($openJobs) > 0) {
            $data = [
                'openJobs' => $openJobs
            ];
        }else{
            $data = [
                'openJobs' => [ null ]
            ];
        }
        return $this->respond($data, 200, 'All Job Openings');
    }

    public function latestJobs(){
        $jobsModel = new JobOpeningsModel();
        $totalJobs = $jobsModel->countAllResults();
        $data = [];
        $openJobs = $jobsModel->where('status', 'active')->orderBy('id','DESC')->findAll(3);
        if(count($openJobs) > 0) {
            $data = [
                'latestJobs' => $openJobs,
                'totalJobs' => $totalJobs
            ];
        }else{
            $data = [
                'latestJobs' => null,
                'totalJobs'=> $totalJobs
            ];
        }
        
        
        return $this->respond($data, 200, 'Latest Job Openings');
    }

    public function job($id)
    {
        $jobsModel = new JobOpeningsModel();
        $jobDetails = $jobsModel->find($id);
        $data = [
            'jobDetails' => $jobDetails
        ];
        return $this->respond($data, 200, 'Job Details');
    }


    public function apply()
    {
        $candidate = new CandidateModel();
        $job_application = new JobApplicationModel();
        $candidateGUID = $candidate->getGUID();
        $photo = $this->request->getFile('candidate_photo');
        $cv = $this->request->getFile('candidate_cv');
        $cover_letter = $this->request->getFile('candidate_cover_letter');
        if ($photo->isValid() && $cv->isValid() && $cover_letter->isValid() && !$photo->hasMoved() && !$cv->hasMoved() && !$cover_letter->hasMoved()) {
            $photoName = $photo->getRandomName();
            $photo->move('./uploads/' . $candidateGUID . '/', $photoName);

            $cvName = $cv->getRandomName();
            $cv->move('./uploads/' . $candidateGUID . '/', $cvName);

            $coverLetterName = $cover_letter->getRandomName();
            $cover_letter->move('./uploads/' . $candidateGUID . '/', $coverLetterName);
        }


        $data = [
            'guid' => $candidateGUID,
            'candidate_name' => $this->request->getPost('candidate_name'),
            'candidate_phone' => $this->request->getPost('candidate_phone'),
            'candidate_email' => $this->request->getPost('candidate_email'),
            'candidate_address' => $this->request->getPost('candidate_address'),
            'candidate_highest_qualification' => $this->request->getPost('candidate_highest_qualification'),
            'candidate_present_emp' => $this->request->getPost('candidate_present_emp'),
            'candidate_present_designation' => $this->request->getPost('candidate_present_designation'),
            'candidate_existing_salary' => $this->request->getPost('candidate_existing_salary'),
            'candidate_expected_salary' => $this->request->getPost('candidate_expected_salary'),
            'candidate_photo' => $photoName,
            'candidate_cv' => $cvName,
            'candidate_cover_letter' => $coverLetterName,
            'status' => 'active'
        ];
        $candidate->save($data);
        $appliedCandidate = $candidate->where('guid', $candidateGUID)->first();
        $applicationData = [
            'position_id' => $this->request->getPost('opening_id'),
            'candidate_id' => $appliedCandidate['id'],
            'applied_date' =>date("Y-m-d"),
            'status' => 'active'
        ];
        $job_application->save($applicationData);
        return $this->respondCreated('Applied for Job');
    }

    public function getAppliedJobs()
    {
        $applications = new JobApplicationModel();
        $candidates = new CandidateModel();
        $jobs = new JobOpeningsModel();

        $details = [];
        $appliedJobs = $applications->where('status', 'active')->findAll();
        foreach($appliedJobs as $appliedJob){
            $candidateDetails = $candidates->find($appliedJob['candidate_id']);
            $jobDetails = $jobs->find($appliedJob['position_id']);
            $details['applications'][] = [
                'candidate' => $candidateDetails,
                'job'=> $jobDetails,
                'application_id' => $appliedJob['id']
            ];
        }

        return $this->respond($details, 200, 'All Applications');
        
    }

    public function getApplicationNumber(){
        $applications = new JobApplicationModel();
        $jobs = new JobOpeningsModel();
        $data = [];
        $openJobs = $jobs->where('status','active')->findAll();
        foreach($openJobs as $openJob){
            $apply = $applications->where('position_id',$openJob['id'])->findAll();
            $data[]=[
                'applications'=> count($apply),
                'jobID'=> $openJob['id']
            ];
        }

        return $this->respond($data,200,'Applications Count');

    }

    public function getApplicationsFromJobID($jobID){
        $applicationsModel = new JobApplicationModel();
        $candidatesModel = new CandidateModel();
        $jobModel = new JobOpeningsModel();
        $data = [];
        $job = $jobModel->find($jobID);
        $applications = $applicationsModel->where('position_id',$jobID)->findAll();
        if($applications){
            foreach($applications as $application){
                $candidate = $candidatesModel->find($application['candidate_id']);
                if($candidate){
                    $data['candidates'][] = [
                        'candidate' => $candidate
                    ];
                }
            }
        }else{
            $data['candidates'] = null;
        }

        $data['job'][] = [$job];
        return $this->respond($data,200,'Applied Jobs');
    }

    public function addNewPosition(){
        $jobsModel = new JobOpeningsModel();
        $data = [
            'position_name' => $this->request->getPost('position_name'),
            'reqd_qualifications' => $this->request->getPost('reqd_qualifications'),
            'responsibilities' => $this->request->getPost('responsibilities'),
            'open_date' => $this->request->getPost('open_date'),
            'open_till' => $this->request->getPost('open_till'),
            'num_openings' => $this->request->getPost('num_openings'),
            'status' => 'active'
        ];

        $jobsModel->save($data);

        return $this->respondCreated('Job Position Created');
    }
}
